package pps.core.system.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import pps.cloud.base.service.BaseBaseStationService;
import pps.cloud.base.service.data.BaseOuIdInput;
import pps.cloud.base.service.data.QueryBaseStationInput;
import pps.cloud.base.service.data.QueryBaseStationOutput;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.sys_dictionary.CreateSysDictionaryInput;
import pps.cloud.system.service.data.sys_dictionary.QuerySysDictionaryViewOutput;
import pps.cloud.system.service.data.sys_organization.GetOuListTreeOutput;
import pps.cloud.system.service.data.sys_organization.SelectRegionFirmInput;
import pps.core.common.constant.BusinessConstant;
import pps.core.common.session.PpsUserSession;
import pps.core.common.session.SysOrganization;
import pps.core.system.cache.SysOrganizationCache;
import pps.core.system.constant.OuLevelConstant;
import pps.core.system.entity.*;
import pps.core.system.mapper.*;
import pps.core.system.service.data.sys_organization.*;
import pps.core.system.service.data.sys_organization_rel.GetOuInfoByConditionInput;
import pps.core.system.service.data.sys_organization_rel.GetSysOrganizationRelInput;
import pps.core.system.service.data.sys_organization_rel.QuerySysOrganizationRelInput;
import pps.core.system.service.data.sys_organization_rel.UpdateSysOrganizationRelInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@XText("组织机构接口")
@XService
public class SysOrganizationService {

    @XText("根据ouName查询组织机构树和场站")
    @XApiPost
    public XListResult<QuerySysOrganizationViewOutputTree> selectOuTreeAndStationByOuNameAndStationName(XContext context, GetSysOrganizationInput input) {
        List<String> ouIds = new ArrayList<>();
        BaseBaseStationService stationService = context.getBean(BaseBaseStationService.class);
        QueryBaseStationInput stationInput = new QueryBaseStationInput();
        //根据模糊场站名称查场站
        stationInput.setStationName(input.getOuName());
        XListResult<QueryBaseStationOutput> listResult = stationService.selectStationAllByStationName(context, stationInput);
        List<QueryBaseStationOutput> stationLikeList = listResult.getResult();
        if (CollUtil.isNotEmpty(stationLikeList)) {
            for (QueryBaseStationOutput baseStationView : stationLikeList) {
                if (CharSequenceUtil.isNotBlank(baseStationView.getOuId())) {
                    ouIds.add(baseStationView.getOuId());
                }
            }
        }
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(BusinessConstant.ONE);
        record.setEndTime(new Date());
        record.setOuIds(ouIds);
        List<SysOrganizationView> viewList = mapper.selectOuByOuIdsAndOuName(record);
        List<QuerySysOrganizationViewOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysOrganizationViewOutputTree.class);
        List<QuerySysOrganizationViewOutputTree> trees = new ArrayList<>();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Integer isSuperAdmin = session.getIsSuperAdmin();
        for (QuerySysOrganizationViewOutputTree tree : list) {
            //如果是超级管理员查所以组织机构
            if (BusinessConstant.ONE.equals(isSuperAdmin)) {
                if (CharSequenceUtil.equals(tree.getParentOuId(), "00000000-0000-0000-0000-000000000000")) {
                    trees.add(tree);
                }
            } else {
                if (CharSequenceUtil.equals(tree.getId(), session.getOuId())) {
                    trees.add(tree);
                }
            }
            //子集
            for (QuerySysOrganizationViewOutputTree it : list) {
                if (CharSequenceUtil.equals(OuLevelConstant.CHANG_ZHAN.getCode(), it.getOuLevel()) && CollUtil.isNotEmpty(stationLikeList)) {
                    List<QuerySysOrganizationViewOutputTree> stationList = new ArrayList<>(stationLikeList.size());
                    for (QueryBaseStationOutput baseStationView : stationLikeList) {
                        if (CharSequenceUtil.equals(it.getId(), baseStationView.getOuId())) {
                            QuerySysOrganizationViewOutputTree outputTree = new QuerySysOrganizationViewOutputTree();
                            outputTree.setId(baseStationView.getId());
                            outputTree.setOuName(baseStationView.getStationName());
                            outputTree.setType("station");
                            stationList.add(outputTree);
                        }
                    }
                    it.setChildren(stationList);
                }
                if (CharSequenceUtil.equals(tree.getId(), it.getParentOuId())) {
                    if (CollUtil.isEmpty(tree.getChildren())) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (CollUtil.isNotEmpty(tree.getChildren())) {
                List<QuerySysOrganizationViewOutputTree> children = tree.getChildren();
                children.forEach(outputTree -> {
                    if (ObjectUtil.isNull(outputTree.getSort())) {
                        outputTree.setSort(999);
                    }
                });
                children.sort(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort));
                tree.setChildren(children);
            }
        }
        trees.forEach(outputTree -> {
            if (ObjectUtil.isNull(outputTree.getSort())) {
                outputTree.setSort(999);
            }
        });
        trees.sort(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort));
        return XListResult.success(trees);
    }

    @XText("根据父级id查询儿子级--最后一级查的是场站")
    @XApiPost
    public XListResult<GetSysOrganizationViewOutput> selectOuInfoOrStationByParentOuId(XContext context, GetSysOrganizationRelInput input) {
        List<SysOrganizationView> viewList = new ArrayList<>();
        if (CharSequenceUtil.isNotBlank(input.getOuLevel()) && CharSequenceUtil.equals(OuLevelConstant.CHANG_ZHAN.getCode(), input.getOuLevel())) {
            if (CharSequenceUtil.isNotBlank(input.getParentOuId())) {
                BaseBaseStationService stationService = context.getBean(BaseBaseStationService.class);
                BaseOuIdInput ouIdInput = new BaseOuIdInput();
                ouIdInput.setOuId(input.getParentOuId());
                XListResult<QueryBaseStationOutput> viewsResult = stationService.selectStationListByOuId(context, ouIdInput);
                List<QueryBaseStationOutput> stationViews = viewsResult.getResult();
                if (CollUtil.isEmpty(stationViews)) {
                    return XListResult.success(Collections.emptyList());
                }
                for (QueryBaseStationOutput stationView : stationViews) {
                    SysOrganizationView organizationView = new SysOrganizationView();
                    organizationView.setId(stationView.getId());
                    organizationView.setOuName(stationView.getStationName());
                    organizationView.setType("station");
                    viewList.add(organizationView);
                }
                List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationViewOutput.class);
                return XListResult.success(outputs);
            }
        }
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Integer isSuperAdmin = session.getIsSuperAdmin();
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setEndTime(new Date());
        record.setIsDeleted(BusinessConstant.ONE);
        if (CharSequenceUtil.isNotBlank(input.getParentOuId())) {
            //如果是超级管理员查所以组织机构
            if (BusinessConstant.ONE.equals(isSuperAdmin) || CharSequenceUtil.equals(OuLevelConstant.YOU_QI_DIAO_KONG_ZHONG_XIN.getCode(), session.getOwnerOrgLevel())) {
                record.setParentOuId("00000000-0000-0000-0000-000000000000");
                viewList = mapper.selectOuSonByParentOuId(record);
            } else {
                record.setId(session.getOuId());
                viewList = mapper.selectOuByOuId(record);
            }
        } else {
            viewList = mapper.selectOuSonByParentOuId(record);
        }
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationViewOutput.class);
        return XListResult.success(outputs);
    }

    @XText("根据用户id查询地区公司")
    @XApiPost
    public XListResult<GetSysOrganizationViewOutput> selectRegionFirmByUserId(XContext context, SelectRegionFirmInput input) {
        String ouId = input.getOuId();
        if (CharSequenceUtil.isBlank(input.getOuId())) {
            PpsUserSession session = context.getSession(PpsUserSession.class);
            ouId = session.getOuId();
        }
        SysOrganizationMapper bean = context.getBean(SysOrganizationMapper.class);
        SysOrganizationEnt ou = bean.selectOne(new QueryWrapper<SysOrganizationEnt>()
                .lambda()
                .eq(SysOrganizationEnt::getId, ouId)
                .eq(SysOrganizationEnt::getIsDeleted, BusinessConstant.ONE));
        if (ObjectUtil.isNull(ou)) {
            return XListResult.success(Collections.emptyList());
        }
        List<GetSysOrganizationViewOutput> outputs = new ArrayList<>();
        //查询当前用户所属组织机构的上级是有油气调控中心
        SysOrganizationViewMapper viewMapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        view.setEndTime(new Date());
        view.setId(ouId);
        view.setOuLevel(OuLevelConstant.YOU_QI_DIAO_KONG_ZHONG_XIN.getCode());
        SysOrganizationView organization = viewMapper.getSysOrganizationByOuLevelAndOuId(view);
        if (OuLevelConstant.ZHUAN_YE_GONG_SI.getCode().equals(ou.getOuLevel()) || organization != null && OuLevelConstant.YOU_QI_DIAO_KONG_ZHONG_XIN.getCode().equals(organization.getOuLevel())) {
            List<SysOrganizationEnt> organizationEnts = bean.selectList(new QueryWrapper<SysOrganizationEnt>()
                    .lambda()
                    .eq(SysOrganizationEnt::getOuLevel, OuLevelConstant.DI_QU_GONG_SI.getCode())
                    .eq(SysOrganizationEnt::getIsDeleted, BusinessConstant.ONE)
                    .orderByAsc(SysOrganizationEnt::getId)
            );
            outputs = XCopyUtils.copyNewList(organizationEnts, GetSysOrganizationViewOutput.class);
        } else {
            view = new SysOrganizationView();
            view.setEndTime(new Date());
            view.setId(ouId);
            view.setOuLevel(OuLevelConstant.DI_QU_GONG_SI.getCode());
            SysOrganizationView organization1 = viewMapper.getSysOrganizationByOuLevelAndOuId(view);
            GetSysOrganizationViewOutput output = new GetSysOrganizationViewOutput();
            XCopyUtils.copyObject(organization1, output);
            outputs.add(output);
        }
        return XListResult.success(outputs);
    }

    @XText("根据ouName查询组织机构树")
    @XApiPost
    public XListResult<QuerySysOrganizationViewOutputTree> selectSysOrganizationTreeByOuName(XContext context, GetSysOrganizationInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(BusinessConstant.ONE);
        record.setEndTime(new Date());
        List<SysOrganizationView> viewList = mapper.selectOuTreeByOuName(record);
        List<QuerySysOrganizationViewOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysOrganizationViewOutputTree.class);
        List<QuerySysOrganizationViewOutputTree> trees = new ArrayList<>();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        Integer isSuperAdmin = session.getIsSuperAdmin();
        for (QuerySysOrganizationViewOutputTree tree : list) {
            //如果是超级管理员查所以组织机构
            if (BusinessConstant.ONE.equals(isSuperAdmin)) {
                if (tree.getParentOuId().equals("00000000-0000-0000-0000-000000000000")) {
                    trees.add(tree);
                }
            } else {
                if (CharSequenceUtil.equals(tree.getId(), session.getOuId())) {
                    trees.add(tree);
                }
            }
            //子集
            for (QuerySysOrganizationViewOutputTree it : list) {
                if (CharSequenceUtil.equals(tree.getId(), it.getParentOuId())) {
                    if (CollUtil.isEmpty(tree.getChildren())) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (CollUtil.isNotEmpty(tree.getChildren())) {
                List<QuerySysOrganizationViewOutputTree> children = tree.getChildren();
                children.forEach(outputTree -> {
                    if (ObjectUtil.isNull(outputTree.getSort())) {
                        outputTree.setSort(999);
                    }
                });
                children.sort(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort));
                tree.setChildren(children);
            }
        }
        trees.forEach(outputTree -> {
            if (ObjectUtil.isNull(outputTree.getSort())) {
                outputTree.setSort(999);
            }
        });
        trees.sort(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort));
        return XListResult.success(trees);
    }

    @XText("根据条件组织机构树")
    @XApiPost
    public XListResult<QuerySysOrganizationViewOutputTree> getOuTreeByCondition(XContext context, GetOuInfoByConditionInput input) {
        List<QuerySysOrganizationViewOutputTree> trees = new ArrayList<>();
        String rootId = null;
        String ouId = null;
        if (ObjectUtil.isNotNull(input.getShowAll()) && input.getShowAll()) {
            rootId = "00000000-0000-0000-0000-000000000000";
        } else {
            PpsUserSession session = context.getSession(PpsUserSession.class);
            ouId = session.getOuId();
        }
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(BusinessConstant.ONE);
        record.setEndTime(new Date());
        List<SysOrganizationView> viewList = mapper.selectOuTreeByOuName(record);
        List<QuerySysOrganizationViewOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysOrganizationViewOutputTree.class);
        for (QuerySysOrganizationViewOutputTree tree : list) {
            if (CharSequenceUtil.isNotBlank(rootId)) {
                if (CharSequenceUtil.equals(tree.getParentOuId(), rootId)) {
                    trees.add(tree);
                }
            } else {
                if (CharSequenceUtil.equals(tree.getId(), ouId)) {
                    trees.add(tree);
                }
            }
            //子集
            for (QuerySysOrganizationViewOutputTree it : list) {
                if (CharSequenceUtil.equals(tree.getId(), it.getParentOuId())) {
                    if (CollUtil.isEmpty(tree.getChildren())) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (CollUtil.isNotEmpty(tree.getChildren())) {
                List<QuerySysOrganizationViewOutputTree> children = tree.getChildren();
                children.forEach(outputTree -> {
                    if (ObjectUtil.isNull(outputTree.getSort())) {
                        outputTree.setSort(999);
                    }
                });
                children.sort(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort));
                tree.setChildren(children);
            }
        }
        trees.forEach(outputTree -> {
            if (ObjectUtil.isNull(outputTree.getSort())) {
                outputTree.setSort(999);
            }
        });
        trees.sort(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort));
        return XListResult.success(trees);
    }

    @XText("根据条件查询组织机构")
    @XApiPost
    public XListResult<GetSysOrganizationViewOutput> getOuInfoByCondition(XContext context, GetOuInfoByConditionInput input) {
        String rootId = null;
        String ouId = null;
        if (ObjectUtil.isNotNull(input.getShowAll()) && input.getShowAll()) {
            rootId = "00000000-0000-0000-0000-000000000000";
        } else {
            PpsUserSession session = context.getSession(PpsUserSession.class);
            ouId = session.getOuId();
        }
        List<SysOrganizationView> viewList;
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setEndTime(new Date());
        record.setIsDeleted(BusinessConstant.ONE);
        if (CharSequenceUtil.isBlank(input.getParentOuId())) {
            //如果是超级管理员查所以组织机构
            if (CharSequenceUtil.isNotBlank(rootId)) {
                record.setParentOuId(rootId);
                viewList = mapper.selectOuSonByParentOuId(record);
            } else {
                record.setId(ouId);
                viewList = mapper.selectOuByOuId(record);
            }
        } else {
            viewList = mapper.selectOuSonByParentOuId(record);
        }
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationViewOutput.class);
        return XListResult.success(outputs);
    }

    @XText("根据父级id查询儿子级")
    @XApiPost
    public XListResult<GetSysOrganizationViewOutput> selectOuInfoByParentOuId(XContext context, GetSysOrganizationRelInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(BusinessConstant.ONE);
        List<SysOrganizationView> viewList;
        if (CharSequenceUtil.isBlank(input.getParentOuId())) {
            PpsUserSession session = context.getSession(PpsUserSession.class);
            Integer isSuperAdmin = session.getIsSuperAdmin();
            //如果是超级管理员查所以组织机构
            if (BusinessConstant.ONE.equals(isSuperAdmin)) {
                record.setParentOuId("00000000-0000-0000-0000-000000000000");
                viewList = mapper.selectOuSonByParentOuId(record);
            } else {
                record.setId(session.getOuId());
                viewList = mapper.selectOuByOuId(record);
            }
        } else {
            viewList = mapper.selectOuSonByParentOuId(record);
        }
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationViewOutput.class);
        return XListResult.success(outputs);
    }

    /**
     * 组织机构管理--动态查询列表
     * GET /system/sys-organization/query-sys-organization-list
     * 接口ID：106823391
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-106823391
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQuerySysOrganizationOutput}>
     */
    @XText("组织机构管理--动态查询列表")
    @XApiGet
    public XListResult<DynamicQuerySysOrganizationOutput> querySysOrganizationList(XContext context, DynamicQuerySysOrganizationInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view;
        if (CharSequenceUtil.isAllBlank(input.getId(), input.getParentOuId(), input.getOuName())) {
            //如果条件都为空,则从登录信息获取条件
            view = new SysOrganizationView();
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (BusinessConstant.ONE.equals(session.getIsSuperAdmin())) {
                view.setParentOuId(BusinessConstant.PARENT_OU_ID);
            } else {
                view.setId(session.getOuId());
            }
        } else {
            view = XCopyUtils.copyNewObject(input, SysOrganizationView.class);
        }
        List<SysOrganizationView> viewList = mapper.selectOrgListDynamicQuery(view);
        List<DynamicQuerySysOrganizationOutput> outputs = XCopyUtils.copyNewList(viewList, DynamicQuerySysOrganizationOutput.class);
        return XListResult.success(outputs);
    }

    /**
     * 组织机构管理--查询组织和线路列表
     * GET /system/sys-organization/query-org-and-line-list
     * 接口ID：108626631
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-108626631
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult}<{@link DynamicQuerySysOrganizationOutput}>
     */
    @XText("组织机构管理--查询组织和线路列表")
    @XApiPost
    public XListResult<DynamicQuerySysOrganizationOutput> queryOrgAndLineList(XContext context, DynamicQuerySysOrganizationInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        List<SysOrganizationView> viewList;
        //单独查组织
        if (CharSequenceUtil.isAllBlank(input.getId(), input.getOuName())) {
            //如果条件都为空,则从登录信息获取条件
            PpsUserSession session = context.getSession(PpsUserSession.class);
            view.setId(session.getOuId());
            viewList = mapper.selectOrgList(view);
        } else if (CharSequenceUtil.isNotBlank(input.getOuName())) {
            view.setOuName(input.getOuName());
            viewList = mapper.selectLineList(view);
        } else {
            view.setId(input.getId());
            viewList = mapper.selectOrgAndLineList(view);
            if (CollUtil.isEmpty(viewList)) {
                //查子线路
                viewList = mapper.selectChildLineList(view);
            }
        }
        return XListResult.success(XCopyUtils.copyNewList(viewList, DynamicQuerySysOrganizationOutput.class));
    }


    /**
     * 查询组织和风力发电机列表
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XListResult }<{@link QueryOrgWindTurbineOutput }>
     */
    @XText("组织机构管理--查询组织和风电站列表")
    @XApiPost
    public XListResult<QueryOrgWindTurbineOutput> queryOrgAndWindTurbineList(XContext context, DynamicQuerySysOrganizationInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        List<SysOrganizationView> viewList;
        //单独查组织
        if (CharSequenceUtil.isAllBlank(input.getId(), input.getOuName())) {
            //如果条件都为空,则从登录信息获取条件
            PpsUserSession session = context.getSession(PpsUserSession.class);
            view.setId(session.getOuId());
            viewList = mapper.selectOrgAndWindTurbineList(view);
        } else if (CharSequenceUtil.isNotBlank(input.getOuName())) {
            view.setOuName(input.getOuName());
            viewList = mapper.selectWindTurbineList(view);
        } else {
            view.setId(input.getId());
            viewList = mapper.selectOrgAndWindStationList(view);
        }
        return XListResult.success(XCopyUtils.copyNewList(viewList, QueryOrgWindTurbineOutput.class));
    }

    @XText("根据父级id查询儿子级-特殊")
    @XApiPost
    public XListResult<GetSysOrganizationViewOutput> selectSpecailOuInfoByParentOuId(XContext context, GetSysOrganizationRelInput input) {
        SysDictionaryViewMapper dictionaryViewMapper = context.getBean(SysDictionaryViewMapper.class);
        SysDictionaryView dictionaryView = new SysDictionaryView();
        dictionaryView.setAlias("sys_inform_org");
        boolean isSpecial = false;
        List<SysDictionaryView> dictionaryViews = dictionaryViewMapper.selectListsByAlias(dictionaryView);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        if (CollUtil.isNotEmpty(dictionaryViews)) {
            SysDictionaryView isExit = dictionaryViews.stream()
                    .filter(item ->
                            DateUtil.compare(item.getEndTime(), new Date()) >= 0
                                    && CharSequenceUtil.equals(item.getRemark(), session.getOuId()))
                    .findFirst()
                    .orElse(null);
            if (ObjectUtil.isNotNull(isExit))
                isSpecial = true;
        }
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setEndTime(new Date());
        record.setIsDeleted(BusinessConstant.ONE);
        List<SysOrganizationView> viewList;
        if (XStringUtils.isEmpty(input.getParentOuId())) {
            Integer isSuperAdmin = session.getIsSuperAdmin();
            //如果是超级管理员查所以组织机构
            if (BusinessConstant.ONE.equals(isSuperAdmin) || isSpecial) {
                record.setParentOuId("00000000-0000-0000-0000-000000000000");
                viewList = mapper.selectOuSonByParentOuId(record);
            } else {
                record.setId(session.getOuId());
                viewList = mapper.selectOuByOuId(record);
            }
        } else {
            viewList = mapper.selectOuSonByParentOuId(record);
        }
        List<GetSysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(viewList, GetSysOrganizationViewOutput.class);
        return XListResult.success(outputs);
    }

    @XText("根据登录用户所属组织机构ID查询组织机构子级树")
    @XApiPost
    public XListResult<QuerySysOrganizationViewOutputTree> getSysOrganizationTreeByUserId(XContext context, GetSysOrganizationInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        PpsUserSession session = context.getSession(PpsUserSession.class);
        view.setUserId(session.getId());
        view.setEndTime(new Date());
        List<SysOrganizationView> viewList = mapper.selectOuAllByUserId(view);
        List<QuerySysOrganizationViewOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysOrganizationViewOutputTree.class);
        List<QuerySysOrganizationViewOutputTree> trees = new ArrayList<>();
        for (QuerySysOrganizationViewOutputTree tree : list) {
            if (CharSequenceUtil.equals(tree.getId(), session.getOuId())) {
                trees.add(tree);
            }
            //子集
            for (QuerySysOrganizationViewOutputTree it : list) {
                if (CharSequenceUtil.equals(tree.getId(), it.getParentOuId())) {
                    if (CollUtil.isEmpty(tree.getChildren())) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (CollUtil.isNotEmpty(tree.getChildren())) {
                List<QuerySysOrganizationViewOutputTree> children = tree.getChildren();
                children.forEach(outputTree -> {
                    if (ObjectUtil.isNull(outputTree.getSort())) {
                        outputTree.setSort(999);
                    }
                });
                children.sort(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort));
                tree.setChildren(children);
            }
        }
        trees.forEach(outputTree -> {
            if (ObjectUtil.isNull(outputTree.getSort())) {
                outputTree.setSort(999);
            }
        });
        trees.sort(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort));
        return XListResult.success(trees);
    }

    @XText("查询组织机构树")
    @XApiPost
    public XListResult<QuerySysOrganizationViewOutputTree> getSysOrganizationTree(XContext context, GetSysOrganizationInput input) {
        //先从缓存中获取
        List<SysOrganizationCache> sysOrganizationCaches = SysOrganizationCache.list(context, BusinessConstant.SYS_ORGANIZATION_CACHE);
        //缓存不存在从数据库获取
        if (CollUtil.isEmpty(sysOrganizationCaches)) {
            List<SysOrganizationCache> caches = selectSysOrganizationTree(context, input);
            List<QuerySysOrganizationViewOutputTree> resultTree = XCopyUtils.copyNewList(caches, QuerySysOrganizationViewOutputTree.class);
            return XListResult.success(resultTree);
        }
        List<QuerySysOrganizationViewOutputTree> resultTree = XCopyUtils.copyNewList(sysOrganizationCaches, QuerySysOrganizationViewOutputTree.class);
        return XListResult.success(resultTree);
    }

    private List<SysOrganizationCache> selectSysOrganizationTree(XContext context, GetSysOrganizationInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(BusinessConstant.ONE);
        record.setEndTime(new Date());
        List<SysOrganizationView> viewList = mapper.selectOrganizationEffective(record);
        List<QuerySysOrganizationViewOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysOrganizationViewOutputTree.class);
        List<QuerySysOrganizationViewOutputTree> trees = new ArrayList<>();
        for (QuerySysOrganizationViewOutputTree tree : list) {
            if (CharSequenceUtil.equals(tree.getParentOuId(), "00000000-0000-0000-0000-000000000000")) {
                trees.add(tree);
            }
            //子集
            for (QuerySysOrganizationViewOutputTree it : list) {
                if (CharSequenceUtil.equals(tree.getId(), it.getParentOuId())) {
                    if (CollUtil.isEmpty(tree.getChildren())) {
                        tree.setChildren(new ArrayList<>());
                    }
                    tree.getChildren().add(it);
                }
            }
            if (CollUtil.isNotEmpty(tree.getChildren())) {
                List<QuerySysOrganizationViewOutputTree> children = tree.getChildren();
                children.forEach(outputTree -> {
                    if (ObjectUtil.isNull(outputTree.getSort())) {
                        outputTree.setSort(999);
                    }
                });
                children.sort(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort));
                tree.setChildren(children);
            }
        }
        trees.forEach(outputTree -> {
            if (ObjectUtil.isNull(outputTree.getSort())) {
                outputTree.setSort(999);
            }
        });
        trees.sort(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort));
        List<SysOrganizationCache> caches = XCopyUtils.copyNewList(trees, SysOrganizationCache.class);
        //放入redis缓存中
        SysOrganizationCache.set(context, BusinessConstant.SYS_ORGANIZATION_CACHE, caches);
        return caches;
    }

    @XText("查询组织机构单节点树")
    @XApiPost
    public XListResult<QuerySysOrganizationViewOutputTree> selectSysOrganizationTreeByParentOuId(XContext context, GetSysOrganizationInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(BusinessConstant.ONE);
        List<SysOrganizationView> viewList = mapper.selectOrganizationByParentOuIdOrOuId(record);
        List<QuerySysOrganizationViewOutputTree> list = XCopyUtils.copyNewList(viewList, QuerySysOrganizationViewOutputTree.class);
        List<QuerySysOrganizationViewOutputTree> trees = new ArrayList<>();
        if (CharSequenceUtil.equals("00000000-0000-0000-0000-000000000000", input.getId())) {
            trees.add(list.get(0));
        } else {
            for (QuerySysOrganizationViewOutputTree tree : list) {
                if (CharSequenceUtil.equals(tree.getId(), input.getId())) {
                    trees.add(tree);
                }
                //子集
                for (QuerySysOrganizationViewOutputTree it : list) {
                    if (CharSequenceUtil.equals(tree.getId(), it.getParentOuId())) {
                        if (CollUtil.isEmpty(tree.getChildren())) {
                            tree.setChildren(new ArrayList<>());
                        }
                        tree.getChildren().add(it);
                    }
                }
                if (CollUtil.isNotEmpty(tree.getChildren())) {
                    List<QuerySysOrganizationViewOutputTree> children = tree.getChildren();
                    children.forEach(outputTree -> {
                        if (ObjectUtil.isNull(outputTree.getSort())) {
                            outputTree.setSort(999);
                        }
                    });
                    children.sort(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort));
                    tree.setChildren(children);
                }
            }
        }
        trees.forEach(outputTree -> {
            if (ObjectUtil.isNull(outputTree.getSort())) {
                outputTree.setSort(999);
            }
        });
        trees.sort(Comparator.comparing(QuerySysOrganizationViewOutputTree::getSort));
        return XListResult.success(trees);
    }

    @XText("查询组织机构列表")
    @XApiPost
    public XPageResult<QuerySysOrganizationViewOutput> querySysOrganizationViewPage(XContext context, QuerySysOrganizationViewInput input) {
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        record.setIsDeleted(BusinessConstant.ONE);
        record.setEndTime(new Date());
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        List<SysOrganizationView> list = mapper.selectSysOrganizationList(record);
        if (CollUtil.isNotEmpty(list)) {
            List<String> parentIds = list.stream().map(SysOrganizationView::getId).collect(Collectors.toList());
            List<SysOrganizationView> childList = mapper.selectCountByParentOuIds(parentIds, new Date());
            Map<String, List<SysOrganizationView>> groupChildList = childList.stream().collect(Collectors.groupingBy(SysOrganizationView::getParentOuId));
            list.forEach(item -> {
                if (groupChildList.containsKey(item.getId()))
                    item.setIsHasChildren(1);
                else
                    item.setIsHasChildren(0);
            });
        }
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        //单位级别
        CreateSysDictionaryInput unitLevelInput = new CreateSysDictionaryInput();
        unitLevelInput.setAlias("UNIT_LEVEL");
        XListResult<QuerySysDictionaryViewOutput> unitLevel = dictService.querySysDictionarysByAlias(context, unitLevelInput);
        //介质类型
        CreateSysDictionaryInput mediaTypesInput = new CreateSysDictionaryInput();
        mediaTypesInput.setAlias("MEDIUM_TYPE");
        XListResult<QuerySysDictionaryViewOutput> mediaTypes = dictService.querySysDictionarysByAlias(context, mediaTypesInput);
        //国家
        CreateSysDictionaryInput countryInput = new CreateSysDictionaryInput();
        countryInput.setAlias("COUNTRIES");
        XListResult<QuerySysDictionaryViewOutput> countrys = dictService.querySysDictionarysByAlias(context, countryInput);
        for (SysOrganizationView view : list) {
            //单位级别
            if (CharSequenceUtil.isNotBlank(view.getOuLevel())) {
                if (CollUtil.isNotEmpty(unitLevel.getResult())) {
                    for (QuerySysDictionaryViewOutput item : unitLevel.getResult()) {
                        if (CharSequenceUtil.equals(view.getOuLevel(), item.getDicKey())) {
                            view.setLevelDicName(item.getDicName());
                        }
                    }
                }
                if (CharSequenceUtil.isNotBlank(view.getLevelDicName())) {
                    view.setLevelDicName(view.getOuLevel());
                }
            }
            //介质类型
            if (CharSequenceUtil.isNotBlank(view.getMediaType())) {
                if (CollUtil.isNotEmpty(mediaTypes.getResult())) {
                    StringBuilder buffer = new StringBuilder();
                    for (String mediaType : CharSequenceUtil.splitToArray(view.getMediaType(), ',')) {
                        for (QuerySysDictionaryViewOutput item : mediaTypes.getResult()) {
                            if (mediaType.equals(item.getDicKey())) {
                                buffer.append(",").append(item.getDicName());
                            }
                        }
                    }
                    if (buffer.length() > 0) {
                        String mediaTypeName = buffer.substring(1);
                        view.setPropertyName(mediaTypeName);
                    }
                }
                if (CharSequenceUtil.isBlank(view.getPropertyName())) {
                    view.setPropertyName(view.getMediaType());
                }
            }
            //国家
            if (CharSequenceUtil.isNotBlank(view.getCountry())) {
                if (CollUtil.isNotEmpty(countrys.getResult())) {
                    for (QuerySysDictionaryViewOutput item : countrys.getResult()) {
                        if (CharSequenceUtil.equals(view.getCountry(), item.getDicKey())) {
                            view.setCountryName(item.getDicName());
                        }
                    }
                }
                if (CharSequenceUtil.isBlank(view.getCountryName())) {
                    view.setCountryName(view.getCountry());
                }
            }
        }
        PageInfo<SysOrganizationView> pageInfo = new PageInfo<>(list);
        List<QuerySysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysOrganizationViewOutput.class);
        outputs.sort(Comparator.comparing(QuerySysOrganizationViewOutput::getSort));
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    /**
     * 组织机构管理--分页列表
     * POST /system/sys-organization/querySysOrganizationViewIncluedOwnerPage
     * 接口ID：106663218
     * 接口地址：https://app.apifox.com/project/3196988/apis/api-106663218
     *
     * @param context 上下文
     * @param input   输入
     * @return {@link XPageResult}<{@link QuerySysOrganizationViewOutput}>
     */
    @XText("查询组织机构列表")
    @XApiPost
    public XPageResult<QuerySysOrganizationViewOutput> querySysOrganizationViewIncluedOwnerPage(XContext context, QuerySysOrganizationViewInput input) {
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        SysOrganizationView record = new SysOrganizationView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        record.setIsDeleted(BusinessConstant.ONE);
        List<SysOrganizationView> list = mapper.selectSysOrganizationList(record);
        if (CollUtil.isEmpty(list) && CharSequenceUtil.isNotBlank(record.getId())) {
            SysOrganizationView owner = mapper.selectOne(record);
            List<SysOrganizationView> parentList = mapper.selectOrganizationBySubOuIds(Collections.singletonList(record.getId()));
            if (CollUtil.isNotEmpty(parentList)) {
                SysOrganizationView parent = parentList.get(0);
                owner.setParentOuId(parent.getParentOuId());
                owner.setPartentName(parent.getPartentName());
            }
            list.add(owner);
        }
        //单位级别
        CreateSysDictionaryInput unitLevelInput = new CreateSysDictionaryInput();
        unitLevelInput.setAlias("UNIT_LEVEL");
        XListResult<QuerySysDictionaryViewOutput> unitLevel = dictService.querySysDictionarysByAlias(context, unitLevelInput);
        unitLevel.throwIfFail();
        if (CollUtil.isNotEmpty(list)) {
            List<String> parentIds = list.stream()
                    .map(SysOrganizationView::getId)
                    .collect(Collectors.toList());
            List<SysOrganizationView> childList = mapper.selectCountByParentOuIds(parentIds, null);
            Map<String, List<SysOrganizationView>> groupChildList = childList.stream()
                    .collect(Collectors.groupingBy(SysOrganizationView::getParentOuId));
            for (SysOrganizationView view : list) {
                if (ObjectUtil.isNull(view.getSort())) {
                    view.setSort(0);
                }
                if (ObjectUtil.isNotNull(groupChildList.get(view.getId()))) {
                    view.setIsHasChildren(1);
                } else {
                    view.setIsHasChildren(0);
                }
                //单位级别
                if (CollUtil.isNotEmpty(unitLevel.getResult())) {
                    view.setLevelDicName(unitLevel.getResult().stream()
                            .filter(row -> CharSequenceUtil.equals(row.getDicKey(), view.getOuLevel()))
                            .map(QuerySysDictionaryViewOutput::getDicName)
                            .findAny()
                            .orElse(null));
                }
            }
        }
        PageInfo<SysOrganizationView> pageInfo = new PageInfo<>(list);
        List<QuerySysOrganizationViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysOrganizationViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("组织机构排序")
    @XApiPost
    public XServiceResult updateSysOrganizationRelSort(XContext context, UpdateSysOrganizationRelInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysOrganizationRelMapper mapper = context.getBean(SysOrganizationRelMapper.class);
            Date date = new Date();
            SysOrganizationRelEnt entity = mapper.selectOne(new QueryWrapper<SysOrganizationRelEnt>()
                    .lambda().eq(SysOrganizationRelEnt::getOuId, input.getId())
                    .le(SysOrganizationRelEnt::getEffectTime, date)
                    .ge(SysOrganizationRelEnt::getEndTime, date));
            if (ObjectUtil.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            Integer entitySort = entity.getSort();
            QueryWrapper<SysOrganizationRelEnt> sortQueryWrapper = new QueryWrapper<>();
            sortQueryWrapper.lambda().eq(SysOrganizationRelEnt::getParentOuId, entity.getParentOuId());
            sortQueryWrapper.lambda().le(SysOrganizationRelEnt::getEffectTime, date);
            sortQueryWrapper.lambda().ge(SysOrganizationRelEnt::getEndTime, date);
            //判断是上移还是下移
            if (CharSequenceUtil.equals(input.getSortFlag(), "up")) {
                //sort-=1;
                sortQueryWrapper.lambda().le(SysOrganizationRelEnt::getSort, entitySort);
                sortQueryWrapper.lambda().orderByDesc(SysOrganizationRelEnt::getSort);
            } else {
                //sort+=1;
                sortQueryWrapper.lambda().ge(SysOrganizationRelEnt::getSort, entitySort);
                sortQueryWrapper.lambda().orderByAsc(SysOrganizationRelEnt::getSort);
            }
            List<SysOrganizationRelEnt> relEnts = mapper.selectList(sortQueryWrapper);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (CollUtil.isNotEmpty(relEnts)) {
                SysOrganizationRelEnt relEnt = relEnts.get(1);
                Integer relEntSort = relEnt.getSort();
                if (CharSequenceUtil.equals(input.getSortFlag(), "up")) {
                    if (ObjectUtil.isNotNull(relEntSort) && relEntSort.equals(entitySort)) {
                        relEntSort -= 1;
                    }
                } else {
                    if (ObjectUtil.isNotNull(relEntSort) && relEntSort.equals(entitySort)) {
                        relEntSort += 1;
                    }
                }
                //上方或下方有数据进行交换序号
                relEnt.setSort(entitySort);
                relEnt.setModifyTime(date);
                relEnt.setModifyById(session.getId());
                relEnt.setModifyByName(session.getUserName());
                mapper.updateById(relEnt);
                entity.setSort(relEntSort);
            }
            entity.setModifyById(session.getId());
            entity.setModifyByName(session.getUserName());
            entity.setModifyTime(date);
            mapper.updateById(entity);
            //更新缓存中组织机构树
            selectSysOrganizationTree(context, new GetSysOrganizationInput());
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("新增组织机构信息")
    @XApiPost
    public XServiceResult createSysOrganization(XContext context, CreateSysOrganizationInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
            SysOrganizationRelMapper relMapper = context.getBean(SysOrganizationRelMapper.class);
            SysOrganizationEnt entity = new SysOrganizationEnt();
            XCopyUtils.copyObject(input, entity);
            Date date = new Date();
            String uuid = UUID.randomUUID().toString();
            entity.setId(uuid);
            entity.setIsDeleted(BusinessConstant.ONE);
            entity.setCreateTime(date);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (ObjectUtil.isNotNull(session)) {
                entity.setCreateById(session.getId());
                entity.setCreateByName(session.getUserName());
            }
            mapper.insert(entity);
            //先修改父级为有子集状态
            if (ObjectUtil.isNotNull(input.getParentOuId())) {
                SysOrganizationRelEnt parent = relMapper.selectOne(new QueryWrapper<SysOrganizationRelEnt>()
                        .lambda()
                        .le(SysOrganizationRelEnt::getEffectTime, new Date())
                        .ge(SysOrganizationRelEnt::getEndTime, new Date())
                        .eq(SysOrganizationRelEnt::getOuId, input.getParentOuId()));
                if (ObjectUtil.isNotNull(parent)) {
                    parent.setIsHasChildren(BusinessConstant.ONE);
                    parent.setModifyTime(new Date());
                    if (ObjectUtil.isNotNull(session)) {
                        parent.setModifyById(session.getId());
                        parent.setModifyByName(session.getUserName());
                    }
                    relMapper.updateById(parent);
                }
            }
            //新增组织机构关系表
            insertSysOrganizationRel(context, date, uuid, input.getParentOuId(), input.getSort(), session);
            //新增组织机构属性表
            insertSysOrganizationProperty(context, date, input.getId(), "GAS", session);
            //更新缓存中组织机构树
            selectSysOrganizationTree(context, new GetSysOrganizationInput());
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("获取当前父级下最大的序号")
    @XApiPost
    public XSingleResult<QuerySysOrganizationRelInput> selectMaxSort(XContext context, GetSysOrganizationRelInput input) {
        SysOrganizationRelViewMapper relViewMapper = context.getBean(SysOrganizationRelViewMapper.class);
        SysOrganizationRelView relView = relViewMapper.selectMaxSort(input.getParentOuId());
        QuerySysOrganizationRelInput output = new QuerySysOrganizationRelInput();
        if (ObjectUtil.isNotNull(relView)) {
            output.setSort(relView.getSort() + 1);
        } else {
            output.setSort(1);
        }
        return XSingleResult.success(output);
    }

    @XText("修改组织机构信息")
    @XApiPost
    public XServiceResult updateSysOrganization(XContext context, UpdateSysOrganizationInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
            SysOrganizationRelMapper relMapper = context.getBean(SysOrganizationRelMapper.class);
            SysOrganizationPropertyMapper propertyMapper = context.getBean(SysOrganizationPropertyMapper.class);
            QueryWrapper<SysOrganizationEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysOrganizationEnt::getId, input.getId());
            SysOrganizationEnt entity = mapper.selectOne(queryWrapper);
            if (ObjectUtil.isNotNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            XCopyUtils.copyObject(input, entity);
            Date date = new Date();
            entity.setModifyTime(date);
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (ObjectUtil.isNotNull(session)) {
                entity.setModifyById(session.getId());
                entity.setModifyByName(session.getUserName());
            }
            mapper.updateById(entity);
            //修改组织机构关系表
            SysOrganizationRelEnt relEntity = relMapper.selectOne(new QueryWrapper<SysOrganizationRelEnt>()
                    .lambda()
                    .eq(SysOrganizationRelEnt::getOuId, input.getId())
                    .le(SysOrganizationRelEnt::getEffectTime, new Date())
                    .ge(SysOrganizationRelEnt::getEndTime, new Date()));
            if (ObjectUtil.isNull(relEntity)) {
                throw new XServiceException("数据有误！");
            }
            try {
                //先修改原父级为无子集状态
                if (CharSequenceUtil.isNotBlank(input.getParentOuId()) && !CharSequenceUtil.equals(input.getParentOuId(), relEntity.getParentOuId())) {
                    SysOrganizationRelEnt originalParent = relMapper.selectOne(new QueryWrapper<SysOrganizationRelEnt>().lambda().le(SysOrganizationRelEnt::getEffectTime, new Date())
                            .ge(SysOrganizationRelEnt::getEndTime, new Date())
                            .eq(SysOrganizationRelEnt::getOuId, relEntity.getParentOuId()));
                    if (ObjectUtil.isNotNull(originalParent)) {
                        originalParent.setIsHasChildren(BusinessConstant.ZERO);
                        originalParent.setModifyTime(new Date());
                        if (ObjectUtil.isNotNull(session)) {
                            originalParent.setModifyById(session.getId());
                            originalParent.setModifyByName(session.getUserName());
                        }
                        relMapper.updateById(originalParent);
                    }
                    //修改新父级为有子集状态
                    SysOrganizationRelEnt nowParent = relMapper.selectOne(new QueryWrapper<SysOrganizationRelEnt>()
                            .lambda()
                            .eq(SysOrganizationRelEnt::getOuId, input.getParentOuId())
                            .le(SysOrganizationRelEnt::getEffectTime, new Date())
                            .ge(SysOrganizationRelEnt::getEndTime, new Date()));
                    if (ObjectUtil.isNotNull(nowParent)) {
                        nowParent.setIsHasChildren(BusinessConstant.ONE);
                        nowParent.setModifyTime(new Date());
                        if (ObjectUtil.isNotNull(session)) {
                            nowParent.setModifyById(session.getId());
                            nowParent.setModifyByName(session.getUserName());
                        }
                        relMapper.updateById(nowParent);
                    }
                    relEntity.setEndTime(date);
                    //新增组织机构关系表
                    insertSysOrganizationRel(context, date, input.getId(), input.getParentOuId(), input.getSort(), session);
                }
                relEntity.setSort(input.getSort());
                relEntity.setModifyTime(date);
                if (ObjectUtil.isNotNull(session)) {
                    relEntity.setModifyById(session.getId());
                    relEntity.setModifyByName(session.getUserName());
                }
                relMapper.updateById(relEntity);
                //修改组织机构属性表
                List<SysOrganizationPropertyEnt> propertyEntitys = propertyMapper.selectList(new QueryWrapper<SysOrganizationPropertyEnt>()
                        .lambda()
                        .eq(SysOrganizationPropertyEnt::getOuId, input.getId())
                        .le(SysOrganizationPropertyEnt::getEffectTime, new Date())
                        .ge(SysOrganizationPropertyEnt::getEndTime, new Date()));
                //组织机构属性数据全部改为失效
                propertyEntitys.forEach(sysOrganizationPropertyEnt -> {
                    sysOrganizationPropertyEnt.setEndTime(date);
                    sysOrganizationPropertyEnt.setModifyTime(date);
                    if (ObjectUtil.isNotNull(session)) {
                        sysOrganizationPropertyEnt.setModifyById(session.getId());
                        sysOrganizationPropertyEnt.setModifyByName(session.getUserName());
                    }
                    propertyMapper.updateById(sysOrganizationPropertyEnt);
                });
                insertSysOrganizationProperty(context, date, input.getId(), "GAS", session);
            } catch (Exception e) {
                throw new XServiceException(e);
            }
            //更新缓存中组织机构树
            selectSysOrganizationTree(context, new GetSysOrganizationInput());
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    @XText("删除组织机构")
    @XApiPost
    public XServiceResult deleteSysOrganization(XContext context, DeleteSysOrganizationInput input) throws XServiceException {
        return XTransactionHelper.begin(context, () -> {
            SysOrganizationMapper mapper = context.getBean(SysOrganizationMapper.class);
            SysUserOrganizationRelMapper userOuRelMapper = context.getBean(SysUserOrganizationRelMapper.class);
            SysOrganizationEnt entity = mapper.selectOne(new QueryWrapper<SysOrganizationEnt>()
                    .lambda()
                    .eq(SysOrganizationEnt::getId, input.getId()));
            if (ObjectUtil.isNull(entity)) {
                return XServiceResult.error(context, XError.NotFound);
            }
            List<SysUserOrganizationRelEnt> relEnts = userOuRelMapper.selectList(new QueryWrapper<SysUserOrganizationRelEnt>()
                    .lambda()
                    .eq(SysUserOrganizationRelEnt::getOuId, entity.getId())
                    .le(true, SysUserOrganizationRelEnt::getEffectTime, new Date())
                    .ge(true, SysUserOrganizationRelEnt::getEndTime, new Date()));
            if (CollUtil.isNotEmpty(relEnts)) {
                return XServiceResult.error(200, "当前组织机构下已绑定人员禁止删除！");
            }
            entity.setModifyTime(new Date());
            PpsUserSession session = context.getSession(PpsUserSession.class);
            if (ObjectUtil.isNotNull(session)) {
                entity.setModifyById(session.getId());
                entity.setModifyByName(session.getUserName());
            }
            entity.setIsDeleted(BusinessConstant.ZERO);
            mapper.updateById(entity);
            //组织机构关系表修改为失效状态
            SysOrganizationRelMapper relmapper = context.getBean(SysOrganizationRelMapper.class);
            SysOrganizationRelEnt relEnt = relmapper.selectOne(new QueryWrapper<SysOrganizationRelEnt>()
                    .lambda()
                    .eq(SysOrganizationRelEnt::getOuId, input.getId())
                    .le(true, SysOrganizationRelEnt::getEffectTime, new Date())
                    .ge(true, SysOrganizationRelEnt::getEndTime, new Date()));
            try {
                relEnt.setEndTime(new Date());
                relEnt.setModifyTime(new Date());
                if (ObjectUtil.isNotNull(session)) {
                    relEnt.setModifyById(session.getId());
                    relEnt.setModifyByName(session.getUserName());
                }
                relmapper.updateById(relEnt);
            } catch (Exception e) {
                throw new XServiceException(e);
            }
            //组织机构属性表修改为失效状态
            SysOrganizationPropertyMapper propertyMapper = context.getBean(SysOrganizationPropertyMapper.class);
            List<SysOrganizationPropertyEnt> propertyEnts = propertyMapper.selectList(new QueryWrapper<SysOrganizationPropertyEnt>()
                    .lambda()
                    .eq(SysOrganizationPropertyEnt::getOuId, input.getId())
                    .le(SysOrganizationPropertyEnt::getEffectTime, new Date())
                    .ge(SysOrganizationPropertyEnt::getEndTime, new Date()));
            propertyEnts.forEach(propertyEnt -> {
                propertyEnt.setEndTime(new Date());
                propertyEnt.setModifyTime(new Date());
                if (ObjectUtil.isNotNull(session)) {
                    propertyEnt.setModifyById(session.getId());
                    propertyEnt.setModifyByName(session.getUserName());
                }
            });
            propertyEnts.forEach(propertyMapper::updateById);
            //更新缓存中组织机构树
            selectSysOrganizationTree(context, new GetSysOrganizationInput());
            //返回执行成功的结果，事务提交
            return XServiceResult.OK;
        });
    }

    //新增组织机构属性表
    public void insertSysOrganizationProperty(XContext context, Date date, String uuid, String mediaTypes, PpsUserSession session) {
        //新增组织机构属性表
        SysOrganizationPropertyMapper propertyMapper = context.getBean(SysOrganizationPropertyMapper.class);
        List<SysOrganizationPropertyEnt> list = new ArrayList<>();
        for (String mediumType : CharSequenceUtil.splitToArray(mediaTypes, ',')) {
            SysOrganizationPropertyEnt propertyEnt = new SysOrganizationPropertyEnt();
            propertyEnt.setId(UUID.randomUUID().toString());
            propertyEnt.setOuId(uuid);
            propertyEnt.setPropertyValue(mediumType);
            propertyEnt.setEffectTime(date);
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String datetime = "9999-12-31";
            try {
                propertyEnt.setEndTime(sdf.parse(datetime));
            } catch (ParseException e) {
                context.getLogger().error(e);
            }
            propertyEnt.setCreateTime(date);
            if (ObjectUtil.isNotNull(session)) {
                propertyEnt.setCreateById(session.getId());
                propertyEnt.setCreateByName(session.getUserName());
            }
            list.add(propertyEnt);
        }
        if (CollUtil.isNotEmpty(list)) {
            list.forEach(propertyMapper::insert);
        }
    }

    //新增组织机构关系表
    public void insertSysOrganizationRel(XContext context, Date date, String ouId, String parentOuId, Integer sort, PpsUserSession session) {
        SysOrganizationRelMapper relMapper = context.getBean(SysOrganizationRelMapper.class);
        SysOrganizationRelViewMapper relViewMapper = context.getBean(SysOrganizationRelViewMapper.class);
        //新增组织机构关系表
        SysOrganizationRelEnt relEnt = new SysOrganizationRelEnt();
        relEnt.setId(UUID.randomUUID().toString());
        relEnt.setOuId(ouId);
        relEnt.setParentOuId(parentOuId);
        SysOrganizationRelEnt relEnt1 = relMapper.selectOne(new QueryWrapper<SysOrganizationRelEnt>()
                .lambda()
                .eq(SysOrganizationRelEnt::getOuId, parentOuId)
                .le(SysOrganizationRelEnt::getEffectTime, date)
                .ge(true, SysOrganizationRelEnt::getEndTime, date));
        if (ObjectUtil.isNull(relEnt1)) {
            relEnt.setOuIdPath(ouId);
        } else {
            relEnt.setOuIdPath(relEnt1.getOuIdPath() + "/" + ouId);
            relEnt.setLevels(ObjectUtil.isNull(relEnt1.getLevels()) ? 1 : relEnt1.getLevels() + 1);
        }
        //如果用户不填写序号需要取表中最大的序号然后+1作为当前数据的序号
        if (ObjectUtil.isNull(sort)) {
            SysOrganizationRelView relView = relViewMapper.selectMaxSort(parentOuId);
            if (ObjectUtil.isNotNull(relView)) {
                relEnt.setSort(relView.getSort() + 1);
            } else {
                relEnt.setSort(1);
            }
        } else {
            relEnt.setSort(sort);
        }
        relEnt.setEffectTime(date);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String datetime = "9999-12-31";
        try {
            relEnt.setEndTime(sdf.parse(datetime));
        } catch (ParseException e) {
            context.getLogger().error(e);
        }
        relEnt.setCreateTime(date);
        if (ObjectUtil.isNotNull(session)) {
            relEnt.setCreateById(session.getId());
            relEnt.setCreateByName(session.getUserName());
        }
        relMapper.insert(relEnt);
        //判断当前组织机构是否存在下级，如果存在下级，更新下级组织机构的路径
        updateSysOrganizationChildRel(context, date, relEnt, session);
    }

    public void updateSysOrganizationChildRel(XContext context, Date date, SysOrganizationRelEnt relEnt, PpsUserSession session) {
        SysOrganizationRelMapper relMapper = context.getBean(SysOrganizationRelMapper.class);
        List<SysOrganizationRelEnt> childList = relMapper.selectList(new QueryWrapper<SysOrganizationRelEnt>()
                .lambda()
                .like(SysOrganizationRelEnt::getOuIdPath, relEnt.getOuId() + "/"));
        if (CollUtil.isNotEmpty(childList)) {
            childList.forEach(item -> {
                int index = item.getOuIdPath().indexOf(relEnt.getOuId());
                item.setOuIdPath(relEnt.getOuIdPath() + item.getOuIdPath().substring(index + relEnt.getOuId().length()));
                item.setModifyById(session.getId());
                item.setModifyByName(session.getUserName());
                item.setLevels(relEnt.getLevels() + 1);
                item.setModifyTime(date);
                relMapper.updateById(item);
            });
        }
    }

    @XText("获取用户组织机构及授权的组织机构")
    @XApiPost
    public XListResult<GetOuListTreeOutput> getOuListWithSelfAndAuth(XContext context, GetOuListWithSelfAndAuthInput input) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        String ouId = session.getOuId();
        if (CharSequenceUtil.isBlank(ouId)) return XListResult.success();
        //根据组织机构id获取所有的子级组织机构
        List<GetOuListTreeOutput> ouList = this.getAllOuListByOuId(context, ouId);
        if (CollUtil.isEmpty(ouList)) return XListResult.success();
        if (CharSequenceUtil.isBlank(input.getMenuId())) {
            return XListResult.success(this.createOuTree(ouList));
        }
        //获取用户授权的组织机构
        Map<String, List<SysOrganization>> pageOrgMap = session.getPageOrgMap();
        if (CollUtil.isNotEmpty(pageOrgMap)) {
            //获取当前用户在指定菜单下被授权的组织机构
            List<SysOrganization> authList = pageOrgMap.get(input.getMenuId());
            if (CollUtil.isNotEmpty(authList)) {
                List<String> authOutIds = new ArrayList<>();
                for (SysOrganization auth : authList) {
                    boolean isAdd = true;
                    for (GetOuListTreeOutput ou : ouList) {
                        if (CharSequenceUtil.equals(ou.getOuId(), auth.getOuId())) {
                            isAdd = false;
                            break;
                        }
                    }
                    if (isAdd) {
                        authOutIds.add(auth.getOuId());
                    }
                }
                if (CollUtil.isNotEmpty(authOutIds)) {
                    List<GetOuListTreeOutput> resultAuthList = this.getOuListByOuIds(context, authOutIds);
                    ouList.addAll(resultAuthList);
                }
            }
        }
        return XListResult.success(this.createOuTree(ouList));
    }

    /**
     * 根据组织机构id获取所有的子级组织机构
     */
    private List<GetOuListTreeOutput> getAllOuListByOuId(XContext context, String ouId) {
        List<GetOuListTreeOutput> outputs = new ArrayList<>();
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        SysOrganizationView view = new SysOrganizationView();
        view.setIsDeleted(1);
        view.setId(ouId);
        List<SysOrganizationView> list = mapper.selectAllListByOuId(view);
        if (CollUtil.isNotEmpty(list)) {
            for (SysOrganizationView item : list) {
                GetOuListTreeOutput out = new GetOuListTreeOutput();
                out.setId(item.getId());
                out.setOuId(item.getId());
                out.setOuName(item.getOuName());
                out.setShortName(item.getShortName());
                out.setParentOuId(item.getParentOuId());
                outputs.add(out);
            }
        }
        return outputs;
    }

    /**
     * 根据组织机构id的集合，获取组织机构信息
     */
    private List<GetOuListTreeOutput> getOuListByOuIds(XContext context, List<String> ouIds) {
        List<GetOuListTreeOutput> outputs = new ArrayList<>();
        if (CollUtil.isEmpty(ouIds)) return outputs;
        SysOrganizationViewMapper mapper = context.getBean(SysOrganizationViewMapper.class);
        List<SysOrganizationView> list = mapper.selectListByOuIds(ouIds);
        if (CollUtil.isNotEmpty(list)) {
            for (SysOrganizationView item : list) {
                GetOuListTreeOutput out = new GetOuListTreeOutput();
                out.setId(item.getId());
                out.setOuId(item.getId());
                out.setOuName(item.getOuName());
                out.setShortName(item.getShortName());
                out.setParentOuId(item.getParentOuId());
                outputs.add(out);
            }
        }
        return outputs;
    }

    public List<GetOuListTreeOutput> createOuTree(List<GetOuListTreeOutput> list) {
        List<GetOuListTreeOutput> treeList = new ArrayList<>();
        if (CollUtil.isNotEmpty(list)) {
            for (GetOuListTreeOutput tree : list) {
                boolean isHaveParent = false;
                for (GetOuListTreeOutput item : list) {
                    //子集
                    if (CharSequenceUtil.equals(tree.getOuId(), item.getParentOuId())) {
                        if (CollUtil.isEmpty(tree.getChildren())) {
                            tree.setChildren(new ArrayList<>());
                        }
                        tree.getChildren().add(item);
                    }
                    //父集
                    if (CharSequenceUtil.equals(tree.getParentOuId(), item.getOuId())) {
                        isHaveParent = true;
                    }
                }
                if (!isHaveParent) {
                    treeList.add(tree);
                }
            }
        }
        return treeList;
    }
}