package pps.core.base.service;


import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;
import org.jsoup.nodes.Element;
import org.jsoup.select.Elements;
import pps.cloud.base.service.BaseWatherCloudService;
import pps.cloud.base.service.data.CrawlBaseWatherInput;
import pps.core.base.entity.BasePhotovoltaicPlantView;
import pps.core.base.entity.BaseWatherEnt;
import pps.core.base.entity.PlantPredictedPowerDataEnt;
import pps.core.base.enums.WindDirection;
import pps.core.base.mapper.BasePhotovoltaicPlantViewMapper;
import pps.core.base.mapper.PlantPredictedPowerDataMapper;
import pps.core.base.mapper.PlantPredictedPowerDataViewMapper;
import pps.core.base.mapper.WeatherDataMapper;
import pps.core.base.utils.HttpUtils;
import pps.core.common.constant.BusinessConstant;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiGet;

import java.math.BigDecimal;
import java.net.URL;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;


@XService
public class BaseWatherCloudServiceImpl implements BaseWatherCloudService {
    final String provinceUrl = "http://weather.cma.cn/api/dict/province";
    final String cityUrl = "http://weather.cma.cn/api/dict/province/";

    public static void insertData(XContext context, String plantId, String provinceCode, String cityCode, int runCount) {
        String url = "http://weather.cma.cn/web/weather/" + cityCode + ".html";
        try {
            List<String> timeList = Arrays.asList("15:00", "30:00", "45:00");
            PlantPredictedPowerDataMapper mapper = context.getBean(PlantPredictedPowerDataMapper.class);
            Document document = Jsoup.parse(new URL(url), 30000);
            Elements elements = document.getElementsByClass("mt15"); // 获取class name 是‘mt15’的都有元素对象
            Element element;
            if (elements.isEmpty()) {
                return;
            } else {
                element = elements.get(1); //根据页面结构获取到需要的元素对象
            }
            Elements tables = element.getElementsByTag("table");
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            List<String> hourTimeArray;// 时间
            List<String> weatherArray; // 天气
            List<String> temperatureArray; // 气温
            List<String> precipitationArray; // 降水
            List<String> windSpeedArray; // 风速
            List<String> windDirectionArray; // 风向
            List<String> pressureArray; // 气压
            List<String> humidityArray;// 湿度
            List<String> cloudCoverArray; // 云量
            Date date;
            Calendar calendar;
            String dayTime;
            List<PlantPredictedPowerDataEnt> objList = new ArrayList<>();
            List<PlantPredictedPowerDataEnt> allList = new ArrayList<>();
            if (!tables.isEmpty()) {
                for (int i = 0; i < tables.size(); i++) {
                    calendar = Calendar.getInstance();
                    calendar.add(Calendar.DATE, i);
                    date = calendar.getTime();
                    dayTime = sdf.format(date);
                    Elements trs = tables.get(i).select("tr");
                    hourTimeArray = new ArrayList<>();// 时间
                    weatherArray = new ArrayList<>(); // 天气
                    temperatureArray = new ArrayList<>(); // 气温
                    precipitationArray = new ArrayList<>(); // 降水
                    windSpeedArray = new ArrayList<>(); // 风速
                    windDirectionArray = new ArrayList<>(); // 风向
                    pressureArray = new ArrayList<>(); // 气压
                    humidityArray = new ArrayList<>(); // 湿度
                    cloudCoverArray = new ArrayList<>(); // 云量
                    if (!trs.isEmpty()) {
                        for (int j = 0; j < trs.size(); j++) { // 循环行
                            Elements tds = trs.get(j).select("td");
                            for (int k = 1; k < tds.size(); k++) { // 循环列
                                Element td = tds.get(k);
                                String td_text;
                                if ("wicon".equals(td.className()))
                                    td_text = td.getElementsByTag("img").attr("src");
                                else
                                    td_text = td.text();
                                if (j == 0)
                                    hourTimeArray.add(td_text);
                                if (j == 1)
                                    weatherArray.add(td_text);
                                if (j == 2)
                                    temperatureArray.add(td_text);
                                if (j == 3)
                                    precipitationArray.add(td_text);
                                if (j == 4)
                                    windSpeedArray.add(td_text);
                                if (j == 5)
                                    windDirectionArray.add(td_text);
                                if (j == 6)
                                    pressureArray.add(td_text);
                                if (j == 7)
                                    humidityArray.add(td_text);
                                if (j == 8)
                                    cloudCoverArray.add(td_text);
                            }
                        }
                    }
                    int Hous = 0;
                    for (int y = 0; y < 8; y++) {
                        PlantPredictedPowerDataEnt dao = new PlantPredictedPowerDataEnt();
                        dao.setCreateTime(new Date());
                        dao.setPlantId(plantId);
                        dao.setHourTime(hourTimeArray.get(y));
                        dao.setTemperature(new BigDecimal(temperatureArray.get(y).replace("℃", "")));
                        dao.setWindSpeed(new BigDecimal(windSpeedArray.get(y).replace("m/s", "")));
                        dao.setWindDirection(new BigDecimal(WindDirection.findByValue(windDirectionArray.get(y)).getValue()));
                        dao.setPressure(new BigDecimal(pressureArray.get(y).replace("hPa", "")));
                        dao.setHumidity(new BigDecimal(humidityArray.get(y).replace("%", "")));
                        //判断小时数与查到的小时数，如果查到的小时数小，判断为第二天
                        if (Hous >= LocalTime.parse(hourTimeArray.get(y)).getHour()) {
                            calendar = Calendar.getInstance();
                            calendar.add(Calendar.DATE, (i + 1));
                            date = calendar.getTime();
                            dayTime = sdf.format(date);
                        }
                        Hous = LocalTime.parse(hourTimeArray.get(y)).getHour();
                        dao.setYearTime(LocalDate.parse(dayTime).getYear() + "");
                        dao.setMonthTime(LocalDate.parse(dayTime).getMonthValue() < 10 ? "0" + LocalDate.parse(dayTime).getMonthValue() : LocalDate.parse(dayTime).getMonthValue() + "");
                        dao.setDayTime(LocalDate.parse(dayTime).getDayOfMonth() < 10 ? "0" + LocalDate.parse(dayTime).getDayOfMonth() : LocalDate.parse(dayTime).getDayOfMonth() + "");
                        dao.setHourTime(Hous < 10 ? "0" + Hous : Hous + "");
                        // 5.执行SQL
                        mapper.delete(new QueryWrapper<PlantPredictedPowerDataEnt>().lambda().eq(PlantPredictedPowerDataEnt::getPlantId, plantId)
                                .eq(PlantPredictedPowerDataEnt::getYearTime, dao.getYearTime())
                                .eq(PlantPredictedPowerDataEnt::getMonthTime, dao.getMonthTime())
                                .eq(PlantPredictedPowerDataEnt::getDayTime, dao.getDayTime())
                                .eq(PlantPredictedPowerDataEnt::getHourTime, dao.getHourTime()));
                        dao.setMinTime("00:00");
                        dao.setDataDate(dao.getYearTime() + "-" + dao.getMonthTime() + "-" + dao.getDayTime() + " " + dao.getHourTime() + ":" + dao.getMinTime());
                        PlantPredictedPowerDataEnt insertDao = XCopyUtils.copyNewObject(dao, PlantPredictedPowerDataEnt.class);
                        allList.add(insertDao);
                        PlantPredictedPowerDataEnt isExit = objList.stream().filter(item -> item.getYearTime().equals(dao.getYearTime()) &&
                                item.getMonthTime().equals(dao.getMonthTime()) &&
                                item.getDayTime().equals(dao.getDayTime()) &&
                                item.getHourTime().equals(dao.getHourTime())).findFirst().orElse(null);
                        if (isExit == null)
                            objList.add(dao);
                    }
                }
                // 原站点时间结构为：08:00	11:00	14:00	17:00	20:00	23:00	02:00	05:00
                // 两条数据中间差了两个小时，使用上下两条数据推算出中间差掉的两个小时
                for (int i = 0; i < objList.size() - 1; i++) {
                    for (int y = 1; y < 3; y++) {
                        PlantPredictedPowerDataEnt dao = new PlantPredictedPowerDataEnt();
                        dao.setCreateTime(new Date());
                        dao.setPlantId(plantId);
                        int I_i = i + 1;
                        BigDecimal temperature_1 = objList.get(i).getTemperature();
                        BigDecimal temperature_2 = objList.get(I_i).getTemperature();
                        dao.setTemperature(temperature_1.add(temperature_2.subtract(temperature_1).multiply(new BigDecimal(0.3 * y))));
                        BigDecimal windspeed_1 = objList.get(i).getWindSpeed();
                        BigDecimal windspeed_2 = objList.get(I_i).getWindSpeed();
                        dao.setWindSpeed(windspeed_1.add(windspeed_2.subtract(windspeed_1).multiply(new BigDecimal(0.3 * y))));
                        dao.setWindDirection(objList.get(i).getWindDirection());
                        BigDecimal pressure_1 = objList.get(i).getPressure();
                        BigDecimal pressure_2 = objList.get(I_i).getPressure();
                        dao.setPressure(pressure_1.add(pressure_2.subtract(pressure_1).multiply(new BigDecimal(0.3 * y))));
                        BigDecimal humidity_1 = objList.get(i).getHumidity();
                        BigDecimal humidity_2 = objList.get(I_i).getHumidity();
                        dao.setHumidity(humidity_1.add(humidity_2.subtract(humidity_1).multiply(new BigDecimal(0.3 * y))));
                        date = sdf.parse(objList.get(i).getYearTime() + "-" + objList.get(i).getMonthTime() + "-" + objList.get(i).getDayTime());
                        String Hour;
                        if (23 == Integer.parseInt(objList.get(i).getHourTime())) {
                            calendar = Calendar.getInstance();
                            calendar.setTime(date);
                            calendar.add(Calendar.DAY_OF_MONTH, 1);
                            date = calendar.getTime();
                            Hour = "0" + (y - 1);
                        } else {
                            if ((Integer.parseInt(objList.get(i).getHourTime()) + y) < 10) {
                                Hour = "0" + (Integer.parseInt(objList.get(i).getHourTime()) + y);
                            } else {
                                Hour = (Integer.parseInt(objList.get(i).getHourTime()) + y) + "";
                            }
                        }
                        dayTime = sdf.format(date);
                        dao.setYearTime(LocalDate.parse(dayTime).getYear() + "");
                        dao.setMonthTime(LocalDate.parse(dayTime).getMonthValue() < 10 ? "0" + LocalDate.parse(dayTime).getMonthValue() : LocalDate.parse(dayTime).getMonthValue() + "");
                        dao.setDayTime(LocalDate.parse(dayTime).getDayOfMonth() < 10 ? "0" + LocalDate.parse(dayTime).getDayOfMonth() : LocalDate.parse(dayTime).getDayOfMonth() + "");
                        dao.setHourTime(Hour);
                        // 5.执行SQL
                        mapper.delete(new LambdaQueryWrapper<PlantPredictedPowerDataEnt>()
                                .eq(PlantPredictedPowerDataEnt::getPlantId, plantId)
                                .eq(PlantPredictedPowerDataEnt::getYearTime, dao.getYearTime())
                                .eq(PlantPredictedPowerDataEnt::getMonthTime, dao.getMonthTime())
                                .eq(PlantPredictedPowerDataEnt::getDayTime, dao.getDayTime())
                                .eq(PlantPredictedPowerDataEnt::getHourTime, dao.getHourTime()));
                        dao.setMinTime("00:00");
                        dao.setDataDate(dao.getYearTime() + "-" + dao.getMonthTime() + "-" + dao.getDayTime() + " " + dao.getHourTime() + ":" + dao.getMinTime());
                        PlantPredictedPowerDataEnt insertDao = XCopyUtils.copyNewObject(dao, PlantPredictedPowerDataEnt.class);
                        allList.add(insertDao);
                    }
                }
                List<PlantPredictedPowerDataEnt> batchList = new ArrayList<>();
                PlantPredictedPowerDataViewMapper dataViewMapper = context.getBean(PlantPredictedPowerDataViewMapper.class);
                List<PlantPredictedPowerDataEnt> sortList = allList.stream().sorted(Comparator.comparing(PlantPredictedPowerDataEnt::getDataDate)).collect(Collectors.toList());
                for (int i = 0; i < sortList.size(); i++) {
                    PlantPredictedPowerDataEnt item = sortList.get(i);
                    batchList.add(item);
                    if (i < sortList.size() - 1) {
                        for (int y = 0; y < timeList.size(); y++) {
                            PlantPredictedPowerDataEnt dao = XCopyUtils.copyNewObject(item);
                            dao.setMinTime(timeList.get(y));
                            dao.setDataDate(dao.getYearTime() + "-" + dao.getMonthTime() + "-" + dao.getDayTime() + " " + dao.getHourTime() + ":" + dao.getMinTime());
                            int I_i = i + 1;
                            BigDecimal temperature_1 = sortList.get(i).getTemperature() == null ? new BigDecimal(0) : sortList.get(i).getTemperature();
                            BigDecimal temperature_2 = sortList.get(I_i).getTemperature() == null ? new BigDecimal(0) : sortList.get(I_i).getTemperature();
                            dao.setTemperature(temperature_1.add(temperature_2.subtract(temperature_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                            BigDecimal humidity_1 = sortList.get(i).getHumidity() == null ? new BigDecimal(0) : sortList.get(i).getHumidity();
                            BigDecimal humidity_2 = sortList.get(I_i).getHumidity() == null ? new BigDecimal(0) : sortList.get(I_i).getHumidity();
                            dao.setHumidity(humidity_1.add(humidity_2.subtract(humidity_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                            BigDecimal windSpeed_1 = sortList.get(i).getWindSpeed() == null ? new BigDecimal(0) : sortList.get(i).getWindSpeed();
                            BigDecimal windSpeed_2 = sortList.get(I_i).getWindSpeed() == null ? new BigDecimal(0) : sortList.get(I_i).getWindSpeed();
                            dao.setWindSpeed(windSpeed_1.add(windSpeed_2.subtract(windSpeed_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                            BigDecimal pressure_1 = sortList.get(i).getPressure() == null ? new BigDecimal(0) : sortList.get(i).getPressure();
                            BigDecimal pressure_2 = sortList.get(I_i).getPressure() == null ? new BigDecimal(0) : sortList.get(I_i).getPressure();
                            dao.setPressure(pressure_1.add(pressure_2.subtract(pressure_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                            BigDecimal planeIrradiance_1 = sortList.get(i).getPlaneIrradiance() == null ? new BigDecimal(0) : sortList.get(i).getPlaneIrradiance();
                            BigDecimal planeIrradiance_2 = sortList.get(I_i).getPlaneIrradiance() == null ? new BigDecimal(0) : sortList.get(I_i).getPlaneIrradiance();
                            dao.setPlaneIrradiance(planeIrradiance_1.add(planeIrradiance_2.subtract(planeIrradiance_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                            BigDecimal horizontalIrradiance_1 = sortList.get(i).getHorizontalIrradiance() == null ? new BigDecimal(0) : sortList.get(i).getHorizontalIrradiance();
                            BigDecimal horizontalIrradiance_2 = sortList.get(I_i).getHorizontalIrradiance() == null ? new BigDecimal(0) : sortList.get(I_i).getHorizontalIrradiance();
                            dao.setHorizontalIrradiance(horizontalIrradiance_1.add(horizontalIrradiance_2.subtract(horizontalIrradiance_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                            BigDecimal power_1 = sortList.get(i).getPower() == null ? new BigDecimal(0) : sortList.get(i).getPower();
                            BigDecimal power_2 = sortList.get(I_i).getPower() == null ? new BigDecimal(0) : sortList.get(I_i).getPower();
                            dao.setPower(power_1.add(power_2.subtract(power_1).multiply(BigDecimal.valueOf(0.3 * (y + 1)))));
                            batchList.add(dao);
                        }
                    }
                    if (batchList.size() > 500) {
                        dataViewMapper.insertBatch(batchList);
                        batchList = new ArrayList<>();
                    }
                }
                if (CollUtil.isNotEmpty(batchList)) {
                    dataViewMapper.insertBatch(batchList);
                }

                //训练接口http://127.0.0.1:10098/aiprediction/xgbtrain?plantId=018a64635ac47cf58895147b0e1bf7e3
                //自动调用预测接口
                HttpUtils.send2("http://127.0.0.1:10098/aiprediction/xgbreason?plantId=" + plantId, "");
            }
        } catch (Exception e) {
            context.getLogger().error(e.getMessage());
        }
    }

    public static void insertData(XContext context, String provinceCode, String provinceName, String cityCode, String cityName, int runCount) {
        String url = "http://weather.cma.cn/web/weather/" + cityCode + ".html";
        try {
            WeatherDataMapper mapper = context.getBean(WeatherDataMapper.class);
            Document document = Jsoup.parse(new URL(url), 30000);
            Elements elements = document.getElementsByClass("mt15"); // 获取class name 是‘mt15’的都有元素对象
            Element element;
            if (elements.isEmpty()) {
                return;
            } else {
                element = elements.get(1); //根据页面结构获取到需要的元素对象
            }
            Elements tables = element.getElementsByTag("table");
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            List<String> hourTimeArray;// 时间
            List<String> weatherArray; // 天气
            List<String> temperatureArray; // 气温
            List<String> precipitationArray; // 降水
            List<String> windSpeedArray; // 风速
            List<String> windDirectionArray; // 风向
            List<String> pressureArray; // 气压
            List<String> humidityArray;// 湿度
            List<String> cloudCoverArray; // 云量
            Date date;
            Calendar calendar;
            String dayTime;
            BaseWatherEnt dao;
            List<BaseWatherEnt> objList = new ArrayList<>();
            if (!tables.isEmpty()) {
                for (int i = 0; i < tables.size(); i++) {
                    calendar = Calendar.getInstance();
                    calendar.add(Calendar.DATE, i);
                    date = calendar.getTime();
                    dayTime = sdf.format(date);
                    Elements trs = tables.get(i).select("tr");
                    hourTimeArray = new ArrayList<>(10);// 时间
                    weatherArray = new ArrayList<>(10); // 天气
                    temperatureArray = new ArrayList<>(10); // 气温
                    precipitationArray = new ArrayList<>(10); // 降水
                    windSpeedArray = new ArrayList<>(10); // 风速
                    windDirectionArray = new ArrayList<>(10); // 风向
                    pressureArray = new ArrayList<>(10); // 气压
                    humidityArray = new ArrayList<>(10); // 湿度
                    cloudCoverArray = new ArrayList<>(10); // 云量
                    if (!trs.isEmpty()) {
                        for (int j = 0; j < trs.size(); j++) { // 循环行
                            Elements tds = trs.get(j).select("td");
                            for (int k = 1; k < tds.size(); k++) { // 循环列
                                Element td = tds.get(k);
                                String td_text;
                                if (CharSequenceUtil.equals("wicon", td.className()))
                                    td_text = td.getElementsByTag("img").attr("src");
                                else
                                    td_text = td.text();
                                if (j == 0)
                                    hourTimeArray.add(td_text);
                                if (j == 1)
                                    weatherArray.add(td_text);
                                if (j == 2)
                                    temperatureArray.add(td_text);
                                if (j == 3)
                                    precipitationArray.add(td_text);
                                if (j == 4)
                                    windSpeedArray.add(td_text);
                                if (j == 5)
                                    windDirectionArray.add(td_text);
                                if (j == 6)
                                    pressureArray.add(td_text);
                                if (j == 7)
                                    humidityArray.add(td_text);
                                if (j == 8)
                                    cloudCoverArray.add(td_text);
                            }
                        }
                    }
                    int Hous = 0;
                    for (int y = 0; y < 8; y++) {
                        dao = new BaseWatherEnt();
                        dao.setCityCode(cityCode);
                        dao.setCityName(cityName);
                        dao.setProvinceCode(provinceCode);
                        dao.setProvinceName(provinceName);
                        dao.setHourTime(hourTimeArray.get(y));
                        dao.setWeather(weatherArray.get(y));
                        dao.setTemperature(temperatureArray.get(y));
                        dao.setPrecipitation(precipitationArray.get(y));
                        dao.setWindSpeed(windSpeedArray.get(y));
                        dao.setWindDirection(windDirectionArray.get(y));
                        dao.setPressure(pressureArray.get(y));
                        dao.setHumidity(humidityArray.get(y));
                        dao.setCloudCover(cloudCoverArray.get(y));
                        //由于界面时间为当日8点到第二日5点数据，所以写了下面的判断分支
                        if (Hous == 0 || Hous <= LocalTime.parse(hourTimeArray.get(y)).getHour()) {
                            dao.setDayTime(dayTime);
                            Hous = LocalTime.parse(hourTimeArray.get(y)).getHour();
                        } else if (2 == LocalTime.parse(hourTimeArray.get(y)).getHour() || 5 == LocalTime.parse(hourTimeArray.get(y)).getHour()) {
                            calendar = Calendar.getInstance();
                            calendar.add(Calendar.DATE, (i + 1));
                            date = calendar.getTime();
                            dayTime = sdf.format(date);
                            dao.setDayTime(dayTime);
                        } else
                            continue;
                        // 5.执行SQL
                        mapper.delete(new LambdaQueryWrapper<BaseWatherEnt>()
                                .eq(BaseWatherEnt::getProvinceCode, dao.getProvinceCode())
                                .eq(BaseWatherEnt::getCityCode, dao.getCityCode())
                                .eq(BaseWatherEnt::getDataDate, dao.getDayTime() + ' ' + dao.getHourTime()));
                        dao.setDataDate(dao.getDayTime() + ' ' + dao.getHourTime());
                        mapper.insert(dao);
                        objList.add(dao);
                    }
                }
                // 原站点时间结构为：08:00	11:00	14:00	17:00	20:00	23:00	02:00	05:00
                // 两条数据中间差了两个小时，使用上下两条数据推算出中间差掉的两个小时
                for (int i = 0; i < objList.size() - 1; i++) {
                    for (int y = 1; y < 3; y++) {
                        dao = new BaseWatherEnt();
                        dao.setCityCode(cityCode);
                        dao.setCityName(cityName);
                        dao.setProvinceCode(provinceCode);
                        dao.setProvinceName(provinceName);
                        dao.setWeather(objList.get(i).getWeather());
                        int I_i = i + 1;
                        double temperature_1 = Double.parseDouble(objList.get(i).getTemperature());
                        double temperature_2 = Double.parseDouble(objList.get(I_i).getTemperature());
                        dao.setTemperature((temperature_1 + (temperature_2 - temperature_1) * (0.3 * y)) + "℃");
                        dao.setPrecipitation(objList.get(i).getPrecipitation());
                        double windspeed_1 = Double.parseDouble(objList.get(i).getWindSpeed());
                        double windspeed_2 = Double.parseDouble(objList.get(I_i).getWindSpeed());
                        dao.setWindSpeed((windspeed_1 + (windspeed_2 - windspeed_1) * (0.3 * y)) + "m/s");
                        dao.setWindDirection(objList.get(i).getWindDirection());
                        double pressure_1 = Double.parseDouble(objList.get(i).getPressure());
                        double pressure_2 = Double.parseDouble(objList.get(I_i).getPressure());
                        dao.setPressure((pressure_1 + (pressure_2 - pressure_1) * (0.3 * y)) + "hPa");
                        double humidity_1 = Double.parseDouble(objList.get(i).getHumidity());
                        double humidity_2 = Double.parseDouble(objList.get(I_i).getHumidity());
                        dao.setHumidity((humidity_1 + (humidity_2 - humidity_1) * (0.3 * y)) + "%");
                        double cloudcover_1 = Double.parseDouble(objList.get(i).getCloudCover());
                        double cloudcover_2 = Double.parseDouble(objList.get(I_i).getCloudCover());
                        dao.setCloudCover((cloudcover_1 + (cloudcover_2 - cloudcover_1) * (0.3 * y)) + "%");
                        if (23 == LocalTime.parse(objList.get(i).getHourTime()).getHour()) {
                            date = sdf.parse(objList.get(i).getDayTime());
                            calendar = Calendar.getInstance();
                            calendar.setTime(date);
                            calendar.add(Calendar.DAY_OF_MONTH, 1);
                            Date tomorrow = calendar.getTime();
                            dao.setDayTime(sdf.format(tomorrow));
                            dao.setHourTime("0" + (y - 1) + ":00:00");
                        } else {
                            dao.setDayTime(objList.get(i).getDayTime());
                            String Hour;
                            if ((LocalTime.parse(objList.get(i).getHourTime()).getHour() + y) < 10) {
                                Hour = "0" + (LocalTime.parse(objList.get(i).getHourTime()).getHour() + y) + ":00:00";
                            } else {
                                Hour = (LocalTime.parse(objList.get(i).getHourTime()).getHour() + y) + ":00:00";
                            }
                            dao.setHourTime(Hour);
                        }
                        // 5.执行SQL
                        mapper.delete(new LambdaQueryWrapper<BaseWatherEnt>().eq(BaseWatherEnt::getProvinceCode, dao.getProvinceCode())
                                .eq(BaseWatherEnt::getCityCode, dao.getCityCode())
                                .eq(BaseWatherEnt::getDataDate, dao.getDayTime() + ' ' + dao.getHourTime()));
                        dao.setDataDate(dao.getDayTime() + ' ' + dao.getHourTime());
                        mapper.insert(dao);
                    }
                }
            }
        } catch (Exception e) {
            context.getLogger().error(e.getMessage());
        }
    }

    @Override
    @XApiGet
    public XServiceResult crawlPlantBaseWatherData(XContext context) {
        try {
            //1.获取电站数据，并根据电站数据，获取到电站所在城市，然后获取到天气数据
            BasePhotovoltaicPlantViewMapper mapper = context.getBean(BasePhotovoltaicPlantViewMapper.class);
            BasePhotovoltaicPlantView view = new BasePhotovoltaicPlantView();
            view.setIsDeleted(BusinessConstant.ONE);
            List<BasePhotovoltaicPlantView> plantList = mapper.selectPlantDetailList(view);
            for (BasePhotovoltaicPlantView item : plantList) {
                if (CharSequenceUtil.isAllNotBlank(item.getCityCode(), item.getProvinceCode())) {
                    insertData(context, item.getId(), item.getProvinceCode(), item.getCityCode(), 1);
                }
            }
        } catch (Exception e) {
            context.getLogger().error(e.getMessage());
        }
        return XServiceResult.OK;
    }

    @Override
    public XServiceResult crawlBaseWatherData(XContext context) {
        try {
            //请求省份接口。获取省份列表
            String provinceStr = HttpUtils.send2(provinceUrl, "");
            //解析省份列表字符串，把字符串解析成json对象
            JSONObject jsonObject = JSONObject.parseObject(provinceStr);
            //处理json对象，变为数组
            String[] provinceList = CharSequenceUtil.splitToArray(jsonObject.getString("data"), '|');
            for (String proStr : provinceList) {
                if (CharSequenceUtil.isNotBlank(proStr)) {
                    String[] pro = CharSequenceUtil.splitToArray(proStr, ',');
                    String provinceCode = pro[0];//省编码
                    String provinceName = pro[1];//省名称
                    //甘肃 .陕西 （全国周期抓取耗费时间超长，共有2400个城市）
                    if (CharSequenceUtil.equals(provinceCode, "AJL")) {
                        //请求省份下的区信息
                        String cityString = HttpUtils.send2(cityUrl + provinceCode, "");
                        //解析区json，把字符串解析成json对象
                        JSONObject cityObj = JSONObject.parseObject(cityString);
                        //处理json对象，变为数组
                        String[] cityList = CharSequenceUtil.splitToArray(cityObj.getString("data"), '|');
                        for (String cityStr : cityList) {
                            if (CharSequenceUtil.isNotBlank(cityStr)) {
                                String[] city = CharSequenceUtil.splitToArray(cityStr, ',');
                                String cityCode = city[0];//区编码
                                String cityName = city[1];//区名称
                                XThreadHelper.async(() -> insertData(context, provinceCode, provinceName, cityCode, cityName, 1));
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            context.getLogger().error(e);
        }
        return XServiceResult.OK;
    }

    @Override
    public XServiceResult crawlBaseWatherDataByCityCode(XContext context, CrawlBaseWatherInput input) {
        insertData(context, input.getProvinceCode(), input.getProvinceName(), input.getCityCode(), input.getCityName(), 1);
        return XServiceResult.OK;
    }

    @XApiGet
    public XServiceResult crawlPlantBaseWatherDataTest(XContext context) {
        crawlPlantBaseWatherData(context);
        return XServiceResult.OK;
    }
}