package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import pps.cloud.base.service.IWindPredictionHistoryCloudService;
import pps.core.base.entity.WindPredictionFutureEnt;
import pps.core.base.entity.WindPredictionHistoryView;
import pps.core.base.mapper.WindPredictionFutureMapper;
import pps.core.base.mapper.WindPredictionHistoryViewMapper;
import pps.core.common.utils.BaseUtils;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;

import java.util.ArrayList;
import java.util.List;

/**
 * 风资源训练集数据Cloud模块
 *
 * @author ZWT
 * @date 2024/08/30 14:07
 */
@XService
public class WindPredictionHistoryCloudServiceImpl implements IWindPredictionHistoryCloudService {

    /**
     * 风资源训练集定时任务
     *
     * @param context 上下文
     * @return {@link XServiceResult }
     */
    @Override
    public XServiceResult windPredictionHistoryJob(XContext context) {
        DateTime startTime = DateUtil.beginOfDay(DateUtil.yesterday());
        DateTime endTime = DateUtil.endOfDay(startTime);
        //取日期范围内已预测数据
        WindPredictionFutureMapper mapper = context.getBean(WindPredictionFutureMapper.class);
        List<WindPredictionFutureEnt> list = mapper.selectList(new LambdaQueryWrapper<WindPredictionFutureEnt>()
                .between(WindPredictionFutureEnt::getDataTime, startTime, endTime)
        );
        if (CollUtil.isNotEmpty(list)) {
            WindPredictionHistoryViewMapper viewMapper = context.getBean(WindPredictionHistoryViewMapper.class);
            //按时间范围删除数据
            viewMapper.deleteByTimeRange(startTime, endTime);
            //todo 数据处理,匹配实际发电量和实际风速
            List<WindPredictionHistoryView> historyList = this.dataProcess(list);
            if (historyList.size() > BaseUtils.BATCH_SIZE) {
                List<List<WindPredictionHistoryView>> subList = BaseUtils.getSubList(historyList);
                subList.forEach(viewMapper::batchInsert);
            } else {
                viewMapper.batchInsert(historyList);
            }
        }
        return XServiceResult.OK;
    }

    /*------------------------- private -------------------------*/

    /**
     * 数据处理
     * todo 后续增加匹配实际发电量和实际风速逻辑
     *
     * @param list 列表
     * @return {@link List }<{@link WindPredictionHistoryView }>
     */
    private List<WindPredictionHistoryView> dataProcess(List<WindPredictionFutureEnt> list) {
        List<WindPredictionHistoryView> historyList = new ArrayList<>(list.size());
        for (WindPredictionFutureEnt future : list) {
            historyList.add(WindPredictionHistoryView.builder()
                    .stationId(future.getStationId())
                    .dataTime(future.getDataTime())
                    .windDirection(future.getWindDirection())
                    .windSpeed(future.getWindSpeed())
                    .airTemperature(future.getAirTemperature())
                    .humidity(future.getHumidity())
                    .pressure(future.getPressure())
                    .actualWindDirection(0)
                    .actualWindSpeed(future.getActualWindSpeed())
                    .actualPower(future.getActualPower())
                    .predictedPower(future.getPredictedPower())
                    .build());
        }
        return historyList;
    }
}