package pps.core.prediction.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import pps.core.common.utils.ExcelUtils;
import pps.core.common.utils.ExportExcelUtils;
import pps.core.prediction.entity.PlantPowerDataEnt;
import pps.core.prediction.entity.PlantPowerDataView;
import pps.core.prediction.enums.ImportFieldDic;
import pps.core.prediction.mapper.PlantPowerDataMapper;
import pps.core.prediction.mapper.PlantPowerDataViewMapper;
import pps.core.prediction.service.data.plant_power_data.ImportFileInput;
import pps.core.prediction.service.data.plant_power_data.QueryPlantPowerDataInput;
import pps.core.prediction.service.data.plant_power_data.QueryPlantPowerDataOutput;
import pps.core.prediction.utils.HttpUtils;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.tool.XStorageTool;
import xstartup.data.XFileResult;
import xstartup.data.XFileType;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiUpload;
import xstartup.feature.mybatis.helper.XMapperHelper;

import java.io.InputStream;
import java.util.*;
import java.util.stream.Collectors;

@XService
@XText("电站实际发电数据")
public class PlantPowerDataService {

    @XApiGet
    public XPageResult<QueryPlantPowerDataOutput> queryPlantPowerDataList(XContext context, QueryPlantPowerDataInput input) {
        PlantPowerDataMapper mapper = context.getBean(PlantPowerDataMapper.class);
        QueryWrapper<PlantPowerDataEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(PlantPowerDataEnt::getPlantId, input.getPlantId());
        if (CharSequenceUtil.isNotBlank(input.getStartTime()))
            queryWrapper.lambda().ge(PlantPowerDataEnt::getDataDate, input.getStartTime());
        if (CharSequenceUtil.isNotBlank(input.getEndTime()))
            queryWrapper.lambda().ge(PlantPowerDataEnt::getDataDate, input.getEndTime());
        queryWrapper.lambda().orderByDesc(PlantPowerDataEnt::getDataDate);
        return XMapperHelper.query(mapper, input, queryWrapper, QueryPlantPowerDataOutput.class);
    }

    @XText("下载电站实际发电数据导入模板")
    @XApiGet
    public XFileResult downloadImportTemplate(XContext context) {
        String fileSavePath = XStorageTool.getAbsolutePath("/temp/excel/导入模板_" + DateUtil.formatDate(DateUtil.date()) + ".xlsx");
        String templateFilePath = "template/import_plant_data_template.xlsx";
        InputStream templateInputStream = this.getClass().getClassLoader().getResourceAsStream(templateFilePath);
        byte[] excelContent = ExportExcelUtils.genSingleExcelFileData(templateInputStream, null);
        try {
            ExportExcelUtils.outputFileData(excelContent, fileSavePath);
        } catch (Exception e) {
            context.getLogger().error(e);
        }
        return XFileResult.success(XFileType.APPLICATION_XLSX, fileSavePath);
    }

    @XText("导入电站实际发电数据")
    @XApiUpload
    public XServiceResult importPlantPowerData(XContext context, ImportFileInput input) {
        List<String> headerList = getExcelHeaderList();
        List<Map<String, Object>> mapList = ExcelUtils.readExcel(input.getFile().getInputStream(), input.getFile().getFileName(), headerList, 0);
        try {
            List<PlantPowerDataView> dataList = new ArrayList<>();
            Date minDate = null;
            Date maxDate = null;
            for (Map<String, Object> objectMap : mapList) {
                PlantPowerDataView dataView = BeanUtil.toBean(objectMap, PlantPowerDataView.class);
                dataView.setPlantId(input.getPlantId());
                if (ObjectUtil.isNotNull(dataView.getDataDate())) {
                    if (ObjectUtil.isNull(minDate)) {
                        minDate = dataView.getDataDate();
                        maxDate = dataView.getDataDate();
                    }
                    if (minDate.compareTo(dataView.getDataDate()) > 0) {
                        minDate = dataView.getDataDate();
                    } else if (maxDate.compareTo(dataView.getDataDate()) < 0) {
                        maxDate = dataView.getDataDate();
                    }
                    dataList.add(dataView);
                }
            }
            if (ObjectUtil.isNull(minDate)) {
                return XServiceResult.error(1000, "日期错误");
            }
            PlantPowerDataMapper mapper = context.getBean(PlantPowerDataMapper.class);
            mapper.delete(new QueryWrapper<PlantPowerDataEnt>()
                    .lambda()
                    .eq(PlantPowerDataEnt::getPlantId, input.getPlantId())
                    .ge(PlantPowerDataEnt::getDataDate, minDate)
                    .le(PlantPowerDataEnt::getDataDate, maxDate));
            PlantPowerDataViewMapper viewMapper = context.getBean(PlantPowerDataViewMapper.class);
            viewMapper.insertBatch(dataList);
            //进行训练
            doTrainData(context, input.getPlantId());
        } catch (Exception e) {
            return XServiceResult.error(-1, e.getMessage());
        }
        return XServiceResult.OK;
    }

    private List<String> getExcelHeaderList() {
        List<Map<String, Object>> fieldList = new ArrayList<>();
        for (ImportFieldDic value : ImportFieldDic.values()) {
            Map<String, Object> map = new HashMap<>();
            map.put("col", value.getCol());
            map.put("sort", value.getSort());
            fieldList.add(map);
        }
        fieldList.sort(Comparator.comparing(o -> ((Integer) o.get("sort"))));
        return fieldList.stream().map(x -> ((String) x.get("col"))).collect(Collectors.toList());
    }

    public void doTrainData(XContext context, String plantId) {
        try {
            Object ret = HttpUtils.send2("http://127.0.0.1:10098/aiprediction/xgbtrain?plantId=" + plantId,
                    "");
            context.getLogger().info(ret.toString());
        } catch (Exception e) {
            throw new RuntimeException("调用训练接口失败");
        }
    }
}